/* compjuga.h
 * Created: Nov. 13, 1998 by Daniel M. German (dmg@csg.uwaterloo.ca)
 * Revised: 
 * Copyright 1998 Daniel M. German (dmg@csg.uwaterloo.ca)
 *
 * This file is part of compjuga, a Spanish verbs conjugator 
 * (see http://csg.uwaterloo.ca/~dmg/compjuga
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: compjuga.h,v 1.1 1998/11/18 21:52:51 dmg Exp $
 * 
 */

#ifndef _COMPJUGA_H_
#define _COMPJUGA_H_



typedef struct 
{
  int iOffset;
  char *szSuffix;
} conjugation_spec;

typedef struct 
{
  int iItemIndex;
  int iOffset;
  char *szSuffix;
} conjugation_irreg_spec;


/* These are the 3 tables of regular verbs: amar, tener, partir */


conjugation_spec AmarTable[] = {
  /* Presente */
  {2, "o"},
  {1, "s"},
  {1, ""},
  {1, "mos"},
  {2, "is"},
  {1, "n"},
  /* Preterito */
  {2,"" },
  {1,"ste"},
  {2,""},
  {1,"mos"},
  {1,"steis"},
  {0,"on"},
  /*Futuro */
  {0,""},
  {0,"s"},
  {0,""},
  {0,"emos"},
  {0,"is"},
  {0,"n"},
  /* Copreterito */
  {1,"ba"},
  {1,"bas"},
  {1,"ba"},
  {2,"bamos"},
  {1,"bais"},
  {1,"ban"},
  /*PosPreterito */
  {0,"a"},
  {0,"as"},
  {0,"a"},
  {0,"amos"},
  {0,"ais"},
  {0,"an"},
  /* Subjuntivo Presente */
  {2,"e"},
  {2,"es"},
  {2,"e"},
  {2,"emos"},
  {2,"is"},
  {2,"en"},
  /* Subjuntivo Preterito 1 */
  { 0,"a"},
  { 0,"as"},
  { 0,"a"},
  { 2,"ramos"},
  { 0,"ais"},
  { 0,"an"},
  /* Subjuntivo Preterito 2 */

  { 1,"se"},
  { 1,"ses"},
  { 1,"se"},
  { 2,"semos"},
  { 1,"seis"},
  { 1,"sen"},


  /* Subjuntivo Futuro */
  {0,"e"},
  {0,"es"},
  {0,"e"},
  {2,"remos"},
  {0,"eis"},
  {0,"en"},
  /* Imperativo */
  {0,""}, /* Non existent */
  {1,""},
  {2,"e"},
  {2,"emos"},
  {1,"d"},
  {2,"en"},

  /* Infinitive */

  {1,"ndo"},
  {1,"do"},
  {1,"do"},

  {0,NULL},


};
  

conjugation_spec TemerTable[] = {
  /*Presente */
  {2,"o"},
  {1,"s"},
  {1,""},
  {1,"mos"},
  {2,"is"},
  {1,"n"},
  /*#Preterito */
  {2,""},
  {2,"iste"},
  {2,"i"},
  {2,"imos"},
  {2,"isteis"},
  {2,"ieron"},
  /*#Futuro */
  {0,""},
  {0,"s"},
  {0,""},
  {0,"emos"},
  {0,"is"},
  {0,"n"},
  /*#Copreterito */
  {2,"a"},
  {2,"as"},
  {2,"a"},
  {2,"amos"},
  {2,"ais"},
  {2,"an"},
  
  /*#PosPreterito */
  {0,"a"},
  {0,"as"},
  {0,"a"},
  {0,"amos"},
  {0,"ais"},
  {0,"an"},
  /* #Subjuntivo #Presente */
  {2,"a"},
  {2,"as"},
  {2,"a"},
  {2,"amos"},
  {2,"is"},
  {2,"an"},
  /* #Subjuntivo#Preterito */
  {2,"iera"},/* | 2,iese"}, */
  {2,"ieras"},/* | 2,ieses"}, */
  {2,"iera"},/* | 2,iese"}, */
  {2,"iramos"}, /* | 2,isemos"}, */
  {2,"ierais"}, /* | 2,ieseis"}, */
  {2,"ieran"}, /* | 2,iesen"}, */
  /*+ Subjuntivo pasado 2*/
  {2,"iese"},
  {2,"ieses"},
  {2,"iese"},
  {2,"isemos"}, 
  {2,"ieseis"}, 
  {2,"iesen"}, 

  /*#Futuro */
  {2,"iere"},
  {2,"ieres"},
  {2,"iere"},
  {2,"iremos"},
  {2,"iereis"},
  {2,"ieren"},
  /*#Imperativo */
  {0,""},
  {1,""},
  {2,"a"},
  {2,"amos"},
  {1,"d"},
  {2,"an"},
  /*infinitivo=temer */
  {2,"iendo"},
  {2,"ido"},
  {2,"ido"},


  {0,NULL},
};

conjugation_spec PartirTable[] = {
  /*#Presente */
  {2,"o"},
  {2,"es"},
  {2,"e"},
  {1,"mos"},
  {2,"s"},
  {2,"en"},
  /*#Preterito */
  {2,""},
  {1,"ste"},
  {1,""},
  {1,"mos"},
  {1,"steis"},
  {1,"eron"},
  /*#Futuro */
  {0,""},
  {0,"s"},
  {0,""},
  {0,"emos"},
  {0,"is"},
  {0,"n"},
  /*#Copreterito */
  {2,"a"},
  {2,"as"},
  {2,"a"},
  {2,"amos"},
  {2,"ais"},
  {2,"an"},
  /*#PosPreterito */
  {0,"a"},
  {0,"as"},
  {0,"a"},
  {0,"amos"},
  {0,"ais"},
  {0,"an"},
  /*#Subjuntivo Presente */
  {2,"a"},
  {2,"as"},
  {2,"a"},
  {2,"amos"},
  {2,"is"},
  {2,"an"},
  
  /*#Subjuntivo Preterito */
  {1,"era"}, /* | 1,ese}, */
  {1,"eras"}, /* | 1,eses}, */
  {1,"era"}, /* | 1,ese}, */
  {1,"ramos"}, /* | 1,semos}, */
  {1,"erais"}, /* | 1,eseis"}, */
  {1,"eran"}, /* | 1,esen"}, */

  /* Subjuntivo pasado 2 */

  {1,"ese"}, /* | 1,ese}, */
  {1,"eses"}, /* | 1,eses}, */
  {1,"ese"}, /* | 1,ese}, */
  {1,"semos"}, /* | 1,semos}, */
  {1,"eseis"}, /* | 1,eseis"}, */
  {1,"esen"}, /* | 1,esen"}, */

  /*#Subjuntivo Futuro */
  {1,"ere"},
  {1,"eres"},
  {1,"ere"},
  {1,"remos"},
  {1,"ereis"},
  {1,"eren"},
  /*#Imperativo */
  {0,""},
  {2,"e"},
  {2,"a"},
  {2,"amos"},
  {1,"d"},
  {2,"an"},
  /* Impersonal */
  {1,"endo"},
  {1,"do"},
  {1,"do"},

  {0,NULL},

};

conjugation_irreg_spec ZarTable[] =
{
   {6,3,"c"},
   {30,3,"ce"},
   {31,3,"ces"},
   {32,3,"ce"},
   {33,3,"cemos"},
   {34,3,"cis"},
   {35,3,"cen"},
   {56,3,"ce"},
   {57,3,"cemos"},
   {59,3,"cen"},
   {-1, 0, NULL},
};

conjugation_irreg_spec CarTable[] =
{
   {6,3,"qu"},
   {30,3,"que"},
   {31,3,"ques"},
   {32,3,"que"},
   {33,3,"quemos"},
   {34,3,"quis"},
   {35,3,"quen"},
   {56,3,"que"},
   {57,3,"quemos"},
   {59,3,"quen"},
   {-1, 0, NULL},
};

conjugation_irreg_spec GarTable[] =
{
   {6,2,"u"},
   {30,2,"ue"},
   {31,2,"ues"},
   {32,2,"ue"},
   {33,2,"uemos"},
   {34,2,"uis"},
   {35,2,"uen"},
   {56,2,"ue"},
   {57,2,"uemos"},
   {59,2,"uen"},
   {-1, 0, NULL},
};

#define NO_CONJUGATIONS 63 /* 6 x 10 tenses + 6 imperative + 3 impersonals + NULL */

typedef char string[80];


typedef struct {
  int iVariants;
  string pszConj[3];
} conjugation;

typedef conjugation* conjugation_table[NO_CONJUGATIONS];

conjugation_spec *Regular_Specs[3]  = {AmarTable,TemerTable,PartirTable};




#define DATA_FILE "/tmp/compjuga.dat"


#endif
