#!/usr/bin/perl

# create_data.pl
#
# Created: Nov. 13, 1998 by Daniel M. German (dmg@csg.uwaterloo.ca)
# Revised: 
# Copyright 1998 Daniel M. German (dmg@csg.uwaterloo.ca)
#
# This file is part of compjuga, a Spanish verbs conjugator 
# (see http://csg.uwaterloo.ca/~dmg/compjuga
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2.
#
# This program file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 675 Mass Ave, Cambridge, MA 02139, USA.
#
# $Id: create_data.pl,v 1.1 1998/11/18 20:44:00 dmg Exp $



require GDBM_File;


#
#   What data structure do I need?
#
#   Family                  unsigned char
#   Number of conjugationns unsigned char
#   Foreach conjugation:    sequence of null terminated strings 
#      ItemIndex              unsigned char
#      Offset                 unsigned char
#      Replacement            
#   Options
#   Comments
#   


$Tenses{PRE}   = 0;
$Tenses{PAS}   = 1;
$Tenses{FUT}   = 2;
$Tenses{COP}   = 3;
$Tenses{POS}   = 4;
$Tenses{PREs}  = 5;
$Tenses{PASs}  = 6;
$Tenses{PASs2} = 7;
$Tenses{FUTs}  = 8;
$Tenses{IMP}   = 9;

$DataFile = $ARGV[0];



die "Use $0 <datafile>\n" if $DataFile eq "";


tie %hash, 'GDBM_File', $DataFile, &GDBM_File::GDBM_WRCREAT, 0600;
           # Use the %hash array.


open (IN, "<data/verbs.txt") || die " unable to open verbs file verbs.txt \n";
while (<IN>)
{
    chop;
    next if (/^#/);
    ($Verb, $Fam, $Note, $Options, $Com) = /([^\-]+)\-([0-9]+)([^>]*)>([^<]*)<(.*)$/;
    
    die " Mmm, this record is wrong ... [$_]\n" if $Fam == 0;

    # Lets just store all them the same as they come :)


    $j=0;

    undef (@Results);
    while ($Com =~ /([^:]+):([^;]+);/)
    {
#	print "[$1][$2]\n";

	$Tiempo = $1;
	$Item = $2;
	$Com = $';
	if ($Tiempo =~ /^([1-6])([A-Za-z]+)$/)
	{
	    $Index = 0;
	    $Persona = $1-1;
	    $Tense = $Tenses{$2};
	}
	elsif ($Tiempo =~ /^([1-6])(PASs2)$/)
	{
	    $Index = 0;
	    $Persona = $1-1;
	    $Tense = $Tenses{$2};
	}
	elsif ($Tiempo =~ /^([1-6])([A-Za-z]+)(2)$/)
	{
	    # Unless it is pasado subjuntivo (which has 2 conjugations)
	    # all the ones with a number suffix are optional
	    #	conjugations issued in that order */
	    $Index = 1;
	    $Persona = $1-1;
	    $Tense = $Tenses{$2};
	}
	elsif ($Tiempo eq "participio")
	{
	    $Index = 0;
	    $Persona = 1;
	    $Tense = 10;
	}
	elsif ($Tiempo eq "gerundio")
	{
	    $Index = 0;
	    $Persona = 0;
	    $Tense = 10;
	}
	elsif ($Tiempo eq "participiocomp")
	{
	    $Index = 0;
	    $Persona = 2;
	    $Tense = 10;
	}
	else
	{
	    die " I don't know what to do [$Tiempo]\n";
	}

	$iIndex = $Tense * 6 + $Persona + $Index * 100;
#	print "$iIndex {$Tense} {$Persona} {$Index} {$Tiempo}-> $Item\n";
	assert($iIndex >= 0, "Invalid index $Index");
	assert($Results[$iIndex] eq "", "[$Item] previus conj for this tense $Index $Results[$iIndex]");

	$Results[$iIndex] = $Item;
	$j++;

	# This has been kind of ugly. So far we have an array of
	# which olds in its first 60 elements, the conjugations 
	# which are irregular.

	# The next 60 are when a verb has more than 2 conjugations
    }

#    print "{$j}\n";

    
    $j=0;
    $ItemsString =  "";
    for ($i=0;$i<200;$i++)
    {
	if ($Results[$i] ne "")
	{
	    $j++;
	    $Temp  = $Results[$i];
	    $k=0;
	    while  (substr($Temp,0,$k) eq substr($Verb,0,$k))
	    {
		$k++;
	    }
	    assert($k>0);
	    $Common = substr($Temp,0,$k-1);
	    $Suffix = substr($Temp,$k-1);
	    $Offset = length($Verb) - length($Common);
	    $CountSuffix{$Suffix}++;
	    $POff = pack "C", $Offset;
	    $PItem = pack "C", $i;
	    $ItemsString .= ($PItem. $POff . $Suffix . "\0");

#	    print "   {$i,$Offset,\"$Suffix\"},\n";
	}
    }
    
    $PFam = pack "C", $Fam;
    $PNumberItems = pack "C", $j;
    
    $Entry = $PFam . $PNumberItems . $ItemsString . $Options . "\0" . $Note . "\0";


#    print "{$Com}\n";
    assert($Com eq "");

    $hash{$Verb} = $Entry;

    $iCount++;
    if (($iCount % 100) == 0)
    {
	printf STDERR "$Verb\n";
    }
#    print "-----\n";
}

#foreach $Temp (keys %CountSuffix)
#{
#    $Size = (length($Temp)+1)*$CountSuffix{$Temp};
#    printf ("%5d:%15s:%10d:\n", $CountSuffix{$Temp},$Temp,$Size);
#    $Total +=$Size;
#}
#

close IN;

# We still have to add the regular verbs and the 71, 72 and 73.
# for that, we add the verbs that are not yet in the hash


open(IN, "<data/ver.fam") ||  die " Unable to open ver.fam file\n";
while (<IN>)
{
    chop;
    next if (/^#/); # Skip comments


    die " Unknonw record $_ " unless /^([a-z]+).*,([0-9]+)$/;
    
    $NewVerb = $1;
    $NewFam = $2;
    if ($hash{$NewVerb} eq "")
    {
	$iCount++;
	if (($iCount % 100) == 0)
	{
	    printf STDERR "$NewVerb\n";
	}
	if ($NewFam ==  3 ||
	    $NewFam ==  4 ||
	    $NewFam ==  5 ||
	    $NewFam ==  71 ||
	    $NewFam ==  72 ||
	    $NewFam ==  73 )
	{
	    $PFam = pack "C", $NewFam;
	    $PNumberItems = pack "C", 0;

	    $Entry = $PFam . $PNumberItems;
	    $hash{$NewVerb} = $Entry;
	}
	else
	{
	    die "New verb $NewVerb [$NewFam] of illegal family :)\n";
	}
    }
    
}
close IN;

print "Total number of verbs added: $iCount\n";
untie %hash ;

exit 0;

sub assert
{
    local ($Exp,$Message) = @_;
    die "Assert[$Verb][$Message]" if !$Exp;
}





